
public interface FinalizablePyObject {
	/**
	 * This interface allows PyObjects to have finalizers.
	 * 
	 * If you are writing a custom PyObject that shall directly extend PyObject and have a
	 * finalizer, you can simply extend PyFinalizableObject and overwrite its __del__-method.
	 * Follow the instructions below, starting at 4).
	 * 
	 * If you want to extend some subclass of PyObject that does not yet implement this
	 * interface (e.g. PyFinalizableInstance already does), you have to take care of
	 * the following steps:
	 * 
	 * 1) Let your subclass implement FinalizablePyObject
	 * 
	 * 2) Let it have a member
	 *    FinalizeTrigger finalizeTrigger;
	 *    
	 * 3) In every constructor initialize this member via
	 *    finalizeTrigger = FinalizeTrigger.makeTrigger(this);
	 * 
	 * 4) Write your __del__-method however you intend it.
	 * 
	 * 
	 * If your __del__-method resurrects the object (Python allows this):
	 * 
	 * 5a) In the block where the resurrection occurs let your __del__-method call
	 *    FinalizeTrigger.resurrectObject(this);
	 * 
	 * If you are not sure, whether a resurrection occurs, e.g. because you call some other
	 * method that you are not in control of:
	 * 
	 * 5b) In the block where the resurrection might occur let your __del__-method call
	 *    FinalizeTrigger.resurrectOnDemand(this);
	 * 
	 * 
	 * Note: Regarding to object resurrection, Jython currently emulates CPython's behavior
	 * prior to 3.4. That means the finalizer __del__ is called every time an object gets gc'ed.
	 * If it is resurrected by __del__ and later collected again, the finalizer is called again.
	 * 
	 * In future verions, i.e. from 3.4 onwards, Jython will change this behavior to that of
	 * CPython 3.4 (and Java), i.e. run __del__ on every object only the first time it is gc'ed.
	 */

	public void __del__();
}
