import org.python.core.Py;
import org.python.core.PyClass;
import org.python.core.PyException;
import org.python.core.PyInstance;
import org.python.core.PyObject;


/**
 * A python class instance with __del__ defined.
 * <p>
 * This is a special class due to performance. Defining
 * finalize() on a class, makes the class a lot slower.
 */
public class PyFinalizableInstance extends PyInstance implements FinalizablePyObject {
	FinalizeTrigger finalizeTrigger;

    public PyFinalizableInstance(PyClass iclass) {
        super(iclass);
        finalizeTrigger = new FinalizeTrigger(this);
    }

    // __del__ method is invoked by finalizeTrigger.
    @Override
    public void __del__() {
        try {
            instclass.__del__.__call__(this);
            //Since instclass.__del__ might resurrect this object, it might be necessary
            //to recreate the FinalizeTrigger (at least prior to Python 3.4):
            FinalizeTrigger.resurrectOnDemand(this);
        } catch (PyException exc) {
            // Try to get the right method description.
            PyObject method = instclass.__del__;
            try {
                method = __findattr__("__del__");
            } catch (PyException e) {
                // nothing we can do
            }
            Py.writeUnraisable(exc, method);
        }
    }
}